@@ -7,16 +7,15 @@
     :license: BSD, see LICENSE for details.
 """
 
-import os
 import sys
 import codecs
-from os import path
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
 
 from sphinx import addnodes
 from sphinx.util import parselinenos
+from sphinx.util.nodes import set_source_info
 
 
 class Highlight(Directive):
@@ -57,14 +56,29 @@ class CodeBlock(Directive):
     final_argument_whitespace = False
     option_spec = {
         'linenos': directives.flag,
+        'emphasize-lines': directives.unchanged_required,
     }
 
     def run(self):
         code = u'\n'.join(self.content)
+
+        linespec = self.options.get('emphasize-lines')
+        if linespec:
+            try:
+                nlines = len(self.content)
+                hl_lines = [x+1 for x in parselinenos(linespec, nlines)]
+            except ValueError, err:
+                document = self.state.document
+                return [document.reporter.warning(str(err), line=self.lineno)]
+        else:
+            hl_lines = None
+
         literal = nodes.literal_block(code, code)
         literal['language'] = self.arguments[0]
         literal['linenos'] = 'linenos' in self.options
-        literal.line = self.lineno
+        if hl_lines is not None:
+            literal['highlight_args'] = {'hl_lines': hl_lines}
+        set_source_info(self, literal)
         return [literal]
 
 
@@ -90,27 +104,16 @@ class LiteralInclude(Directive):
         'end-before': directives.unchanged_required,
         'prepend': directives.unchanged_required,
         'append': directives.unchanged_required,
+        'emphasize-lines': directives.unchanged_required,
     }
 
     def run(self):
         document = self.state.document
-        filename = self.arguments[0]
         if not document.settings.file_insertion_enabled:
             return [document.reporter.warning('File insertion disabled',
                                               line=self.lineno)]
         env = document.settings.env
-        if filename.startswith('/') or filename.startswith(os.sep):
-            rel_fn = filename[1:]
-        else:
-            docdir = path.dirname(env.doc2path(env.docname, base=None))
-            rel_fn = path.join(docdir, filename)
-        try:
-            fn = path.join(env.srcdir, rel_fn)
-        except UnicodeDecodeError:
-            # the source directory is a bytestring with non-ASCII characters;
-            # let's try to encode the rel_fn in the file system encoding
-            rel_fn = rel_fn.encode(sys.getfilesystemencoding())
-            fn = path.join(env.srcdir, rel_fn)
+        rel_filename, filename = env.relfn2path(self.arguments[0])
 
         if 'pyobject' in self.options and 'lines' in self.options:
             return [document.reporter.warning(
@@ -120,7 +123,7 @@ class LiteralInclude(Directive):
         encoding = self.options.get('encoding', env.config.source_encoding)
         codec_info = codecs.lookup(encoding)
         try:
-            f = codecs.StreamReaderWriter(open(fn, 'U'),
+            f = codecs.StreamReaderWriter(open(filename, 'rb'),
                     codec_info[2], codec_info[3], 'strict')
             lines = f.readlines()
             f.close()
@@ -137,7 +140,7 @@ class LiteralInclude(Directive):
         objectname = self.options.get('pyobject')
         if objectname is not None:
             from sphinx.pycode import ModuleAnalyzer
-            analyzer = ModuleAnalyzer.for_file(fn, '')
+            analyzer = ModuleAnalyzer.for_file(filename, '')
             tags = analyzer.find_tags()
             if objectname not in tags:
                 return [document.reporter.warning(
@@ -152,7 +155,22 @@ class LiteralInclude(Directive):
                 linelist = parselinenos(linespec, len(lines))
             except ValueError, err:
                 return [document.reporter.warning(str(err), line=self.lineno)]
-            lines = [lines[i] for i in linelist]
+            # just ignore nonexisting lines
+            nlines = len(lines)
+            lines = [lines[i] for i in linelist if i < nlines]
+            if not lines:
+                return [document.reporter.warning(
+                    'Line spec %r: no lines pulled from include file %r' %
+                    (linespec, filename), line=self.lineno)]
+
+        linespec = self.options.get('emphasize-lines')
+        if linespec:
+            try:
+                hl_lines = [x+1 for x in parselinenos(linespec, len(lines))]
+            except ValueError, err:
+                return [document.reporter.warning(str(err), line=self.lineno)]
+        else:
+            hl_lines = None
 
         startafter = self.options.get('start-after')
         endbefore  = self.options.get('end-before')
@@ -179,14 +197,15 @@ class LiteralInclude(Directive):
         text = ''.join(lines)
         if self.options.get('tab-width'):
             text = text.expandtabs(self.options['tab-width'])
-        retnode = nodes.literal_block(text, text, source=fn)
-        retnode.line = 1
-        retnode.attributes['line_number'] = self.lineno
+        retnode = nodes.literal_block(text, text, source=filename)
+        set_source_info(self, retnode)
         if self.options.get('language', ''):
             retnode['language'] = self.options['language']
         if 'linenos' in self.options:
             retnode['linenos'] = True
-        document.settings.env.note_dependency(rel_fn)
+        if hl_lines is not None:
+            retnode['highlight_args'] = {'hl_lines': hl_lines}
+        env.note_dependency(rel_filename)
         return [retnode]
 
 
